@@ -38,7 +38,7 @@ class OutlineItemDictionary(PDFObject):
         struct_elem: StructElem = None,
     ):
         super().__init__()
-        self.title = PDFString(title)
+        self.title = PDFString(title, encrypt=True)
         self.parent = None
         self.prev = None
         self.next = None
