@@ -9,14 +9,18 @@ from .enums import SignatureFlag
 from .errors import FPDFException
 from .outline import build_outline_objs
 from .sign import Signature, sign_content
-from .syntax import build_obj_dict, Name, PDFArray, PDFContentStream, PDFObject
+from .syntax import (
+    build_obj_dict,
+    Name,
+    PDFArray,
+    PDFContentStream,
+    PDFDate,
+    PDFObject,
+    PDFString,
+)
 from .syntax import create_dictionary_string as pdf_dict
 from .syntax import create_list_string as pdf_list
 from .syntax import iobj_ref as pdf_ref
-from .util import (
-    enclose_in_parens,
-    format_date,
-)
 
 from fontTools import ttLib
 from fontTools import subset as ftsubset
@@ -45,6 +49,7 @@ class PDFHeader(ContentWithoutID):
     def __init__(self, pdf_version):
         self.pdf_version = pdf_version
 
+    # method override
     def serialize(self, _security_handler=None):
         return f"%PDF-{self.pdf_version}"
 
@@ -91,11 +96,11 @@ class PDFFontDescriptor(PDFObject):
 
 
 class CIDSystemInfo(PDFObject):
-    def __init__(self, registry, ordering, supplement):
+    def __init__(self):
         super().__init__()
-        self.registry = enclose_in_parens(registry)
-        self.ordering = enclose_in_parens(ordering)
-        self.supplement = supplement
+        self.registry = PDFString("Adobe", encrypt=True)
+        self.ordering = PDFString("UCS", encrypt=True)
+        self.supplement = 0
 
 
 class PDFInfo(PDFObject):
@@ -107,15 +112,15 @@ class PDFInfo(PDFObject):
         keywords,
         creator,
         producer,
-        creation_date,
+        creation_date: PDFDate,
     ):
         super().__init__()
-        self.title = enclose_in_parens(title) if title else None
-        self.subject = enclose_in_parens(subject) if subject else None
-        self.author = enclose_in_parens(author) if author else None
-        self.keywords = enclose_in_parens(keywords) if keywords else None
-        self.creator = enclose_in_parens(creator) if creator else None
-        self.producer = enclose_in_parens(producer) if producer else None
+        self.title = PDFString(title) if title else None
+        self.subject = PDFString(subject) if subject else None
+        self.author = PDFString(author) if author else None
+        self.keywords = PDFString(keywords) if keywords else None
+        self.creator = PDFString(creator) if creator else None
+        self.producer = PDFString(producer) if producer else None
         self.creation_date = creation_date
 
 
@@ -124,8 +129,12 @@ class AcroForm:
         self.fields = fields
         self.sig_flags = sig_flags
 
-    def serialize(self):
-        obj_dict = build_obj_dict({key: getattr(self, key) for key in dir(self)})
+    def serialize(self, _security_handler=None, _obj_id=None):
+        obj_dict = build_obj_dict(
+            {key: getattr(self, key) for key in dir(self)},
+            _security_handler=_security_handler,
+            _obj_id=_obj_id,
+        )
         return pdf_dict(obj_dict, field_join=" ")
 
 
@@ -135,7 +144,7 @@ class PDFCatalog(PDFObject):
     ):
         super().__init__()
         self.type = Name("Catalog")
-        self.lang = enclose_in_parens(lang) if lang else None
+        self.lang = PDFString(lang) if lang else None
         self.page_layout = page_layout
         self.page_mode = page_mode
         self.viewer_preferences = viewer_preferences
@@ -166,7 +175,7 @@ class PDFFontStream(PDFContentStream):
 
 class PDFXmpMetadata(PDFContentStream):
     def __init__(self, contents):
-        super().__init__(contents=contents)
+        super().__init__(contents=contents.encode("utf-8"))
         self.type = Name("Metadata")
         self.subtype = Name("XML")
 
@@ -183,7 +192,6 @@ class PDFXObject(PDFContentStream):
         "height",
         "color_space",
         "bits_per_component",
-        "filter",
         "decode",
         "decode_parms",
         "s_mask",
@@ -214,6 +222,27 @@ class PDFXObject(PDFContentStream):
         self.s_mask = None
 
 
+class PDFICCPObject(PDFContentStream):
+    __slots__ = (  # RAM usage optimization
+        "_id",
+        "_contents",
+        "filter",
+        "length",
+        "n",
+        "alternate",
+    )
+
+    def __init__(
+        self,
+        contents,
+        n,
+        alternate,
+    ):
+        super().__init__(contents=contents, compress=True)
+        self.n = n
+        self.alternate = Name(alternate)
+
+
 class PDFPage(PDFObject):
     __slots__ = (  # RAM usage optimization
         "_id",
@@ -330,6 +359,7 @@ class OutputProducer:
     def __init__(self, fpdf):
         self.fpdf = fpdf
         self.pdf_objs = []
+        self.iccp_i_to_pdf_i = {}
         self.obj_id = 0  # current PDF object number
         # array of PDF object offsets in self.buffer, used to build the xref table:
         self.offsets = {}
@@ -468,10 +498,6 @@ class OutputProducer:
         self.pdf_objs.append(pdf_obj)
         if trace_label:
             self.trace_labels_per_obj_id[self.obj_id] = trace_label
-        if isinstance(pdf_obj, PDFContentStream) and self.fpdf._security_handler:
-            if not isinstance(pdf_obj.content_stream(), (bytearray, bytes)):
-                pdf_obj._contents = pdf_obj.content_stream().encode("latin-1")
-            pdf_obj.encrypt(self.fpdf._security_handler)
         return self.obj_id
 
     def _add_pages_root(self):
@@ -546,7 +572,7 @@ class OutputProducer:
                 # recalcTimestamp=False means that it doesn't modify the "modified" timestamp in head table
                 # if we leave recalcTimestamp=True the tests will break every time
                 fonttools_font = ttLib.TTFont(
-                    file=font["ttffile"], recalcTimestamp=False
+                    file=font["ttffile"], recalcTimestamp=False, fontNumber=0, lazy=True
                 )
 
                 # 1. get all glyphs in PDF
@@ -554,6 +580,7 @@ class OutputProducer:
                 glyph_names = [
                     cmap[unicode] for unicode in uni_to_new_code_char if unicode in cmap
                 ]
+                print(glyph_names)
 
                 missing_glyphs = [
                     chr(unicode)
@@ -669,9 +696,7 @@ class OutputProducer:
                 self._add_pdf_obj(to_unicode_obj, "fonts")
                 composite_font_obj.to_unicode = to_unicode_obj
 
-                cid_system_info_obj = CIDSystemInfo(
-                    registry="Adobe", ordering="UCS", supplement=0
-                )
+                cid_system_info_obj = CIDSystemInfo()
                 self._add_pdf_obj(cid_system_info_obj, "fonts")
                 cid_font_obj.c_i_d_system_info = cid_system_info_obj
 
@@ -708,15 +733,44 @@ class OutputProducer:
                 img_objs_per_index[img["i"]] = self._add_image(img)
         return img_objs_per_index
 
+    def _ensure_iccp(self, img_info):
+        """
+        Returns the PDF object of the ICC profile indexed iccp_i in the FPDF object.
+        Adds it if not present.
+        """
+        iccp_i = img_info["iccp_i"]
+        if iccp_i in self.iccp_i_to_pdf_i:
+            return self.iccp_i_to_pdf_i[iccp_i]
+        iccp_content = None
+        for iccp_c, i in self.fpdf.icc_profiles.items():
+            if iccp_i == i:
+                iccp_content = iccp_c
+                break
+        assert iccp_content is not None
+        # Note: n should be 4 if the profile ColorSpace is CMYK
+        iccp_obj = PDFICCPObject(
+            contents=iccp_content, n=img_info["dpn"], alternate=img_info["cs"]
+        )
+        iccp_pdf_i = self._add_pdf_obj(iccp_obj, "iccp")
+        self.iccp_i_to_pdf_i[iccp_i] = iccp_pdf_i
+        return iccp_pdf_i
+
     def _add_image(self, info):
         color_space = Name(info["cs"])
         decode = None
+        iccp_i = info.get("iccp_i")
         if color_space == "Indexed":
             color_space = PDFArray(
                 ["/Indexed", "/DeviceRGB", f"{len(info['pal']) // 3 - 1}"]
             )
+        elif iccp_i is not None:
+            iccp_pdf_i = self._ensure_iccp(info)
+            color_space = PDFArray(["/ICCBased", str(iccp_pdf_i), str("0"), "R"])
         elif color_space == "DeviceCMYK":
             decode = "[1 0 1 0 1 0 1 0]"
+            raise NotImplementedError(
+                "fpdf2 does not support DeviceCMYK ColorSpace yet - cf. issue #711"
+            )
 
         decode_parms = f"<<{info['dp']} /BitsPerComponent {info['bpc']}>>"
         img_obj = PDFXObject(
@@ -832,9 +886,7 @@ class OutputProducer:
     def _add_xmp_metadata(self):
         if not self.fpdf.xmp_metadata:
             return None
-        xpacket = f'<?xpacket begin="ï»¿" id="W5M0MpCehiHzreSzNTczkc9d"?>\n{self.fpdf.xmp_metadata}\n<?xpacket end="w"?>\n'.encode(
-            "latin-1"
-        )
+        xpacket = f'<?xpacket begin="ï»¿" id="W5M0MpCehiHzreSzNTczkc9d"?>\n{self.fpdf.xmp_metadata}\n<?xpacket end="w"?>\n'
         pdf_obj = PDFXmpMetadata(xpacket)
         self._add_pdf_obj(pdf_obj)
         return pdf_obj
@@ -844,7 +896,7 @@ class OutputProducer:
         creation_date = None
         if fpdf.creation_date:
             try:
-                creation_date = format_date(fpdf.creation_date, with_tz=True)
+                creation_date = PDFDate(fpdf.creation_date, with_tz=True, encrypt=True)
             except Exception as error:
                 raise FPDFException(
                     f"Could not format date: {fpdf.creation_date}"
@@ -912,7 +964,7 @@ class OutputProducer:
             catalog_obj.mark_info = pdf_dict({"/Marked": "true"})
         if fpdf.embedded_files:
             file_spec_names = [
-                f"{enclose_in_parens(embedded_file.basename())} {embedded_file.file_spec().serialize()}"
+                f"{PDFString(embedded_file.basename()).serialize()} {embedded_file.file_spec().serialize()}"
                 for embedded_file in fpdf.embedded_files
                 if embedded_file.globally_enclosed
             ]
