@@ -9,6 +9,7 @@ except ImportError:
     warnings.warn(
         "defusedxml could not be imported - fpdf2 will not be able to sanitize SVG images provided"
     )
+    # nosemgrep: python.lang.security.use-defused-xml.use-defused-xml
     from xml.etree.ElementTree import fromstring as parse_xml_str  # nosec
 
 from . import html
@@ -635,7 +636,8 @@ class SVGObject:
     def __init__(self, svg_text):
         self.cross_references = {}
 
-        svg_tree = parse_xml_str(svg_text)
+        # disabling bandit rule as we use defusedxml:
+        svg_tree = parse_xml_str(svg_text)  # nosec B314
 
         if svg_tree.tag not in xmlns_lookup("svg", "svg"):
             raise ValueError(f"root tag must be svg, not {svg_tree.tag}")
